@@ -49,14 +49,16 @@ class Object_Display:public Gtk_GLArea
      void setupgl();
      void setup_pick(gdouble x, gdouble y);
      void drawgl();
- 
-     gint press(GdkEventButton*); 
     
    public:
      Object_Display();
      ~Object_Display();
-     gint expose(GdkEventExpose* expose);
-     gint resize(GdkEventConfigure *event);
+
+     // take over the impl functions for events 
+     virtual void realize_impl();
+     virtual gint expose_event_impl(GdkEventExpose* expose);
+     virtual gint configure_event_impl(GdkEventConfigure *event);
+     virtual gint button_press_event_impl(GdkEventButton*); 
  
      // callbacks we will connect to other widgets
      void toggle_cube(Gtk_ToggleButton *button);
@@ -94,26 +96,20 @@ Object_Display::Object_Display():Gtk_GLArea()
      //don't get every point of motion
    set_events(GDK_EXPOSURE_MASK|
               GDK_BUTTON_PRESS_MASK);
-
-   connect_to_method(expose_event,this,&expose);
-   connect_to_method(configure_event,this,&resize);
-   connect_to_method(realize,this,&initgl);
-
-   connect_to_method(button_press_event,this,&press);
   }
 
 
-gint Object_Display::resize(GdkEventConfigure *event)
+void Object_Display::realize_impl()
   {
-   if (begingl())
-     {
-      setupgl();
-      endgl();
-     }
-   return TRUE;
+   // we must call the real realize_impl so the object is 
+   // marked realized
+   Gtk_GLArea::realize_impl();
+
+   // then we can call our init
+   initgl();
   }
 
-gint Object_Display::expose(GdkEventExpose *event)
+gint Object_Display::expose_event_impl(GdkEventExpose *event)
   {
    // we are CPU intensive so we should wait for the last event
    if (event->count>0) return TRUE;
@@ -121,6 +117,16 @@ gint Object_Display::expose(GdkEventExpose *event)
    return TRUE;
   }
 
+gint Object_Display::configure_event_impl(GdkEventConfigure *event)
+  {
+   if (make_current())
+     {
+      setupgl();
+     }
+   return TRUE;
+  }
+
+
 // Function has not been test (as far as I know)
 Object_Display::~Object_Display()
   {if (glIsList(cube)==GL_TRUE)
@@ -140,15 +146,16 @@ Object_Display::~Object_Display()
 ***********************************************************/
 void Object_Display::initgl()
   {GLUquadricObj *obj;
+
    // In order to allow the window to be shrunk we will 
    // reset the minumum size to something smaller now 
    // that the inital allocation has been completed.
    //  (this works, but is this what they intended??)
    set_usize(100,100);
 
-   // begingl must be called before performing GL operations
-   // and GL code should only be called if begingl returns true
-   if (begingl())
+   // make_current must be called before performing GL operations
+   // and GL code should only be called if make_current returns true
+   if (make_current())
      {
       setupgl();
 
@@ -211,9 +218,6 @@ void Object_Display::initgl()
       gluCylinder(obj,0,1.3,0.05,15,1);
       gluDeleteQuadric(obj);
       glEndList();
-
-      // endgl must be called to restore the gtk settings.
-      endgl();
      }
   }
 
@@ -344,8 +348,8 @@ void Object_Display::drawgl()
    static GLfloat red[]=  {1.0,0.3,0.3,1.0};
 
    // start opengl rendering, you must put all opengl calls
-   // inside begingl/endgl pairs
-   if (begingl())
+   // inside make_current
+   if (make_current())
      {
       glMatrixMode(GL_MODELVIEW);
       glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
@@ -370,12 +374,9 @@ void Object_Display::drawgl()
         draw_cylinder();
 
       glPopMatrix();
-      endgl();
      }
    
-   // for some reason the swap buffers must be called outside
-   // the endgl command. (else it killed the idle function)
-   swapbuffers();
+   swap_buffers();
   }
 
 /***********************************************************
@@ -396,10 +397,10 @@ void Object_Display::drawgl()
   glGetIntegerv(GL_MODELVIEW_STACK_DEPTH,&i); cout << "Model: "<< i<<endl;
   glGetIntegerv(GL_PROJECTION_STACK_DEPTH,&i); cout << "Proj: "<< i<<endl;
 ***********************************************************/
-gint Object_Display::press(GdkEventButton *event) 
+gint Object_Display::button_press_event_impl(GdkEventButton *event) 
   {GLuint selectBuf[12];
    int i,hits=0;
-   if (begingl())
+   if (make_current())
      {
       // set up the selection buffer (4 ints per hit)
       glSelectBuffer(12, selectBuf);
@@ -432,7 +433,6 @@ gint Object_Display::press(GdkEventButton *event)
 
       // switch to render mode, the return value is the number of hits.
       hits=glRenderMode(GL_RENDER);
-      endgl();
      }
 
    // process the selection buffer
@@ -440,9 +440,12 @@ gint Object_Display::press(GdkEventButton *event)
      {for (i=0;i<hits;i++)
         {
          switch (selectBuf[(i*4)+3])
-           {case 1: set_cube(flag_cube=!flag_cube); break;
-            case 2: set_sphere(flag_sphere=!flag_sphere); break;
-            case 3: set_cylinder(flag_cylinder=!flag_cylinder); break;
+           {case 1: set_cube(flag_cube=!flag_cube); 
+                    break;
+            case 2: set_sphere(flag_sphere=!flag_sphere); 
+                    break;
+            case 3: set_cylinder(flag_cylinder=!flag_cylinder);
+                    break;
            }
         }
     //  drawgl();
@@ -453,17 +456,17 @@ gint Object_Display::press(GdkEventButton *event)
 
 // call backs that we will connect the buttons to.
 void Object_Display::toggle_cube(Gtk_ToggleButton *button)
-  {flag_cube=button->get_state();
+  {flag_cube=button->get_active();
    drawgl();
   }
 
 void Object_Display::toggle_sphere(Gtk_ToggleButton *button)
-  {flag_sphere=button->get_state();
+  {flag_sphere=button->get_active();
    drawgl();
   }
 
 void Object_Display::toggle_cylinder(Gtk_ToggleButton *button)
-  {flag_cylinder=button->get_state();
+  {flag_cylinder=button->get_active();
    drawgl();
   }
 
@@ -476,7 +479,15 @@ void Object_Display::toggle_cylinder(Gtk_ToggleButton *button)
 Gtk_Main *app;
 
 class MainWindowClass : public Gtk_Window 
-  {
+  {private:
+     // Pointers to all of the widgets
+     Gtk_Button *button;
+     Gtk_ToggleButton *button_cube;
+     Gtk_ToggleButton *button_sphere;
+     Gtk_ToggleButton *button_cylinder;
+     Gtk_Frame *frame;
+     Gtk_Table *table;
+     Object_Display *display;
    public:
       MainWindowClass (void);
       ~MainWindowClass (void);
@@ -496,16 +507,6 @@ class MainWindowClass : public Gtk_Window
 ***********************************************************/
 MainWindowClass::MainWindowClass(void) : Gtk_Window(GTK_WINDOW_TOPLEVEL)
   {
-   // Pointers to all of the widgets
-   Gtk_Button *button;
-   Gtk_ToggleButton *button_cube;
-   Gtk_ToggleButton *button_sphere;
-   Gtk_ToggleButton *button_cylinder;
-   Gtk_Frame *frame;
-   Gtk_Table *table;
-   Object_Display *display;
-
-
    set_title("Pick Me");
  
    // Create the OpenGL object (the whole point of this exercise) 
@@ -516,51 +517,51 @@ MainWindowClass::MainWindowClass(void) : Gtk_Window(GTK_WINDOW_TOPLEVEL)
    // Gtk_GLArea can't have a border, so we will stick it in a frame
    frame = new Gtk_Frame();
    frame->set_shadow_type (GTK_SHADOW_IN);
-   frame->border_width(10);  
+   frame->set_border_width(10);  
    frame->add(display);   
    frame->show();
 
    // Create a Quit button
    button_cube = new Gtk_ToggleButton("Cube");
    button_cube->set_usize(100,50);
-   button_cube->border_width(5);
+   button_cube->set_border_width(5);
    button_cube->show();
 
    button_sphere = new Gtk_ToggleButton("Sphere");
    button_sphere->set_usize(100,50);
-   button_sphere->border_width(5);
+   button_sphere->set_border_width(5);
    button_sphere->show();
 
    button_cylinder = new Gtk_ToggleButton("Cylinder");
    button_cylinder->set_usize(100,50);
-   button_cylinder->border_width(5);
+   button_cylinder->set_border_width(5);
    button_cylinder->show();
 
    // Create a Quit button
    button = new Gtk_Button("Quit");
    button->set_usize(100,50);
-   button->border_width(10);
+   button->set_border_width(10);
    button->show();
 
    // Place the Scrollbars and Button in a Table
    // I was using attach_defaults, but it is better if just the scrollbars 
    // grow, to fill the space.
    table = new Gtk_Table(2,4,FALSE);
-   table->attach(frame,0,1,0,4,GTK_EXPAND|GTK_FILL);
-   table->attach(button_cube,1,2,0,1,GTK_SHRINK);
-   table->attach(button_sphere,1,2,1,2,GTK_SHRINK);
-   table->attach(button_cylinder,1,2,2,3,GTK_SHRINK);
-   table->attach(button,1,2,3,4,GTK_SHRINK,GTK_SHRINK,0,0);
+   table->attach(*frame,0,1,0,4,GTK_EXPAND|GTK_FILL);
+   table->attach(*button_cube,1,2,0,1,GTK_SHRINK);
+   table->attach(*button_sphere,1,2,1,2,GTK_SHRINK);
+   table->attach(*button_cylinder,1,2,2,3,GTK_SHRINK);
+   table->attach(*button,1,2,3,4,GTK_SHRINK,GTK_SHRINK,0,0);
    table->set_col_spacing(0,10);
    table->set_row_spacing(2,10);
-   table->border_width(10);
+   table->set_border_width(10);
    table->show();
 
    // Add the box to the mainwindow
    add(table);
 
    // Connect the Quit button to the application quit
-   connect_to_method(button->clicked,(Gtk_Main*)app,&Gtk_Main::quit);
+   connect_to_method(button->clicked,app,&Gtk_Main::quit);
 
    // Connect buttons to the OpenGL object
    connect_to_method(button_cube->toggled,display,
@@ -572,15 +573,23 @@ MainWindowClass::MainWindowClass(void) : Gtk_Window(GTK_WINDOW_TOPLEVEL)
  
    // Connect the OpenGL object back to the buttons 
    connect_to_method(display->set_cube,button_cube,
-                     &Gtk_ToggleButton::set_state);
+                     &Gtk_ToggleButton::set_active);
    connect_to_method(display->set_sphere,button_sphere,
-                     &Gtk_ToggleButton::set_state);
+                     &Gtk_ToggleButton::set_active);
    connect_to_method(display->set_cylinder,button_cylinder,
-                     &Gtk_ToggleButton::set_state);
+                     &Gtk_ToggleButton::set_active);
   }
 
 MainWindowClass::~MainWindowClass(void)
-  {}
+  {
+   delete button;
+   delete button_cube;
+   delete button_sphere;
+   delete button_cylinder;
+   delete frame;
+   delete table;
+   delete display;
+  };
 
 /***********************************************************
 ***** Main 
